#pragma once

#include <endian.h>

template<class super>
class header_seq : public super
{
    typedef typename super::buffer_type buffer;
    typedef uint32_t seq_type;

    struct header_seq_type
    {
        seq_type seq;
    };

    constexpr static size_t m_header_seq_len = sizeof(struct header_seq_type);
    seq_type m_seq_out = 0;
    seq_type m_seq_in = 0;

    header_seq_type *cast_header(buffer &buf)
    {
        return reinterpret_cast<header_seq_type *>(buf.head());
    }

    void write_seq(buffer &buf)
    {
        buf.head_add(m_header_seq_len);
        cast_header(buf)->seq = htobe32(m_seq_out);
    }

    void read_seq(buffer &buf)
    {
        m_seq_in = be32toh(cast_header(buf)->seq);
        buf.head_del(m_header_seq_len);
    }

  public:
    // read sequence header from buffer and forward the buffer up
    bool recv_buf(buffer &buf)
    {
        if (!super::recv_buf(buf))
            return false;

        read_seq(buf);

        return true;
    }

    // add sequence header to buffer and forward the buffer down
    bool send_buf(buffer &buf)
    {
        write_seq(buf);

        if (!super::send_buf(buf))
            return false;

        ++m_seq_out;

        return true;
    }

    seq_type seq_out() const
    {
        return m_seq_out;
    }

    seq_type seq_in() const
    {
        return m_seq_in;
    }
};
